==> jsonapi_links-1.0.8.tar.gz: jsonapi_links/src/EventSubscriber/ResponseSubscriber.php <==
<?php

namespace Drupal\jsonapi_links\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ImmutableConfig;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * EventSubscriber for removing links from JSON:API responses.
 */
class ResponseSubscriber implements EventSubscriberInterface {

  /**
   * The paths to process.
   *
   * @var array
   */
  protected array $path = [];

  /**
   * A list of regexes defining patterns to ignore for removal.
   *
   * @var array
   */
  protected array $ignoreList = [];

  /**
   * The JSON:API Link module's config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Creates a ResponseSubscriber for JSON:API Links module.
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('config.factory'),
      $container->get('current_route_match'),
    );
  }

  public function __construct(ConfigFactoryInterface $config_factory, CurrentRouteMatch $current_route_match) {
    $this->config = $config_factory->get('jsonapi_links');
    $this->currentRouteMatch = $current_route_match;
  }

  /**
   * Checks for a JSON:API response and removes links.
   */
  public function onResponse(ResponseEvent $event) {
    $response = $event->getResponse();
    // First we check if we have an JSON:API response.
    if (strtolower($response->headers->get('content-type', 'unknown')) !== 'application/vnd.api+json') {
      return;
    }

    $content = $response->getContent();
    $json = json_decode($content);

    // 2nd check if we have an JSON:API JSON
    if (!isset($json->jsonapi)) {
      return;
    }

    // This check shall prevent, that the meta.links.me info is removed.
    $jsonapi_base_path = $this->currentRouteMatch->getParameter('jsonapi.base_path');
    if ($event->getRequest()->getPathInfo() === $jsonapi_base_path) {
      return;
    }

    // Check if we shall do anything at all.
    if (!$this->config->get('remove_links')) {
      return;
    }

    if ($this->config->get('ignore_list')) {
      $this->ignoreList = array_filter(array_map('trim', explode("\n", $this->config->get('ignore_list'))));
    }

    $this->removeLinks($json);

    $response->setContent(json_encode($json));
    $event->setResponse($response);
  }

  /**
   * Unsets links in JSON:API responses.
   */
  protected function removeLinks(&$data, ?string $node = NULL) {
    if ($node) {
      $this->path[] = $node;
    }

    // If the current path is on the ignore list, skip everything below that.
    if ($this->isOnIgnoreList()) {
      array_pop($this->path);
      return;
    }

    if (is_scalar($data)) {
      array_pop($this->path);
      return;
    }

    if (is_object($data)) {
      // If this is the JSON:API response, do not remove the links.
      // The links on the response may include pager links.
      if (!property_exists($data, 'jsonapi')) {
        // Meta must return an object. If links are the only property,
        // and the links are removed by this module, then an empty array
        // will be returned instead.
        // So, if meta only contains links, then remove meta as well.
        if (property_exists($data, 'meta') && property_exists($data->meta, 'links')) {
          unset($data->meta->links);
          $property_list = get_object_vars($data->meta);
          if (count($property_list) === 0) {
            unset($data->meta);
          }
        }
        unset($data->links);
      }

      foreach (get_object_vars($data) as $name => $attribute) {
        $this->removeLinks($data->$name);
      }

      array_pop($this->path);
      return;
    }

    if (is_array($data)) {
      foreach ($data as $i => &$item) {
        $this->removeLinks($item, $i);
      }
    }

    array_pop($this->path);
  }

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE][] = ['onResponse', 112];
    return $events;
  }

  /**
   * Checks whether a path is on the ignore list.
   */
  private function isOnIgnoreList(): bool {
    $path = implode('.', $this->path);

    for ($i = 0; $i < count($this->ignoreList); $i++) {
      if (preg_match($this->ignoreList[$i], $path)) {
        return TRUE;
      }
    }

    return FALSE;
  }

}

==> jsonapi_links-1.0.9.tar.gz: jsonapi_links/src/EventSubscriber/ResponseSubscriber.php <==
<?php

namespace Drupal\jsonapi_links\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ImmutableConfig;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * EventSubscriber for removing links from JSON:API responses.
 */
class ResponseSubscriber implements EventSubscriberInterface {

  /**
   * The paths to process.
   *
   * @var array
   */
  protected array $path = [];

  /**
   * A list of regexes defining patterns to ignore for removal.
   *
   * @var array
   */
  protected array $ignoreList = [];

  /**
   * The JSON:API Link module's config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Creates a ResponseSubscriber for JSON:API Links module.
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('config.factory'),
      $container->get('current_route_match'),
    );
  }

  public function __construct(ConfigFactoryInterface $config_factory, CurrentRouteMatch $current_route_match) {
    $this->config = $config_factory->get('jsonapi_links.settings');
    $this->currentRouteMatch = $current_route_match;
  }

  /**
   * Checks for a JSON:API response and removes links.
   */
  public function onResponse(ResponseEvent $event) {
    $response = $event->getResponse();
    // First we check if we have an JSON:API response.
    if (strtolower($response->headers->get('content-type', 'unknown')) !== 'application/vnd.api+json') {
      return;
    }

    $content = $response->getContent();
    $json = json_decode($content);

    // 2nd check if we have an JSON:API JSON
    if (!isset($json->jsonapi)) {
      return;
    }

    // This check shall prevent, that the meta.links.me info is removed.
    $jsonapi_base_path = $this->currentRouteMatch->getParameter('jsonapi.base_path');
    if ($event->getRequest()->getPathInfo() === $jsonapi_base_path) {
      return;
    }

    // Check if we shall do anything at all.
    if (!$this->config->get('remove_links')) {
      return;
    }

    if ($this->config->get('ignore_list')) {
      $this->ignoreList = array_filter(array_map('trim', explode("\n", $this->config->get('ignore_list'))));
    }

    $this->removeLinks($json);

    $response->setContent(json_encode($json));
    $event->setResponse($response);
  }

  /**
   * Unsets links in JSON:API responses.
   */
  protected function removeLinks(&$data, string|int|null $node = NULL) {
    if ($node) {
      $this->path[] = $node;
    }

    // If the current path is on the ignore list, skip everything below that.
    if ($this->isOnIgnoreList()) {
      array_pop($this->path);
      return;
    }

    if (is_scalar($data)) {
      array_pop($this->path);
      return;
    }

    if (is_object($data)) {
      // If this is the JSON:API response, do not remove the links.
      // The links on the response may include pager links.
      if (!property_exists($data, 'jsonapi')) {
        // Meta must return an object. If links are the only property,
        // and the links are removed by this module, then an empty array
        // will be returned instead.
        // So, if meta only contains links, then remove meta as well.
        if (property_exists($data, 'meta') && property_exists($data->meta, 'links')) {
          unset($data->meta->links);
          $property_list = get_object_vars($data->meta);
          if (count($property_list) === 0) {
            unset($data->meta);
          }
        }
        unset($data->links);
      }

      foreach (get_object_vars($data) as $name => $attribute) {
        $this->removeLinks($data->$name);
      }

      array_pop($this->path);
      return;
    }

    if (is_array($data)) {
      foreach ($data as $i => &$item) {
        $this->removeLinks($item, $i);
      }
    }

    array_pop($this->path);
  }

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE][] = ['onResponse', 112];
    return $events;
  }

  /**
   * Checks whether a path is on the ignore list.
   */
  private function isOnIgnoreList(): bool {
    $path = implode('.', $this->path);

    for ($i = 0; $i < count($this->ignoreList); $i++) {
      if (preg_match($this->ignoreList[$i], $path)) {
        return TRUE;
      }
    }

    return FALSE;
  }

}
